@@ -9,8 +9,10 @@ __all__ = ['Configuration']
 DEFAULT_SETTINGS = dict(
     DEBUG=True,
     DEFAULT_LOG_FMT="[%(asctime)s %(levelname)s] %(message)s",
+    DEFAULT_WORKER_LOG_FMT="[%(asctime)s %(levelname)s] [%(process)d] %(message)s",
     LOGGING_LEVEL=logging.DEBUG,
     LOG_REQUESTS=False,
+    LOG_TORNADO_REQUESTS=False,
     LOG_FILE=None,
     LOG_FORCE_COLORIZED=False,
     REDIRECT_STDOUT=True,
@@ -23,6 +25,7 @@ DEFAULT_SETTINGS = dict(
     WORKER_TYPE='sync',
     WORKERS=1,
     CONCURRENCY=1,
+    TORNADO_ENDPOINTS=[],
 )
 
 
