@@ -1,62 +1,117 @@
 """Manage process pool."""
 from __future__ import absolute_import
 
+import json
 import logging
 import sys
-import struct
-import cPickle as pickle
+import os
 
+from gaffer.http.util import CorsHandler
+from gaffer.http_handler import HttpHandler, HttpEndpoint
 from pyuv import Pipe
 from six import iteritems
 
 from thriftworker.utils.loop import in_loop
 from thriftworker.utils.decorators import cached_property
 from thriftworker.utils.mixin import LoopMixin
+
+from thriftpool.app import current_app
+from thriftpool.exceptions import SystemTerminate
 from thriftpool.utils.mixin import LogsMixin
+from thriftpool.utils.serializers import StreamSerializer
+from thriftpool.rpc.manager import Clients
 
 from .base import StartStopComponent
-from .proto import Producer
 
 logger = logging.getLogger(__name__)
 
 
+#: How long we should wait for process initialization.
+START_TIMEOUT = 60.0
+
+#: How long we should wait for process termination.
+STOP_TIMEOUT = 60.0
+
+
+class BaseHandler(CorsHandler):
+
+    def initialize(self, clients):
+        self._clients = clients
+
+    def _execute(self, *args, **kwargs):
+        current_app.hub.spawn(super(BaseHandler, self)._execute, *args, **kwargs)
+
+
+class ClientsHandler(BaseHandler):
+
+    def get(self):
+        self.preflight()
+        self.set_status(200)
+        self.write(json.dumps(self._clients.keys()))
+
+
+class BaseClientHandler(BaseHandler):
+
+    def get_data(self, proxy):
+        raise NotImplementedError('subclass responsibility')
+
+    def get(self, *args):
+        self.preflight()
+
+        try:
+            pid = int(args[0])
+        except ValueError:
+            self.set_status(400)
+            self.write({"error": "bad_value"})
+            return
+
+        if pid in self._clients:
+            self.set_status(200)
+        else:
+            self.set_status(404)
+            return
+
+        client = self._clients[pid]
+        data = client.spawn(self.get_data).get()
+        self.write(json.dumps(data))
+
+
+class CounterHandler(BaseClientHandler):
+
+    def get_data(self, proxy):
+        return {'{0}.{1}'.format(service, method): value
+                for (service, method), value in proxy.get_counters().items()}
+
+
+class TimerHandler(BaseClientHandler):
+
+    def get_data(self, proxy):
+        return {'{0}.{1}'.format(service, method): value
+                for (service, method), value in proxy.get_timers().items()}
+
+
 class RedirectStream(object):
     """Try to write to stream asynchronous."""
 
     def __init__(self, loop, stream):
+        self.fd = None
         self.stream = stream
         try:
-            fd = stream.fileno()
+            fd = self.fd = stream.fileno()
         except AttributeError:
             self.channel = None
         else:
             channel = self.channel = Pipe(loop)
+            setattr(channel, 'bypass', True)
             channel.open(fd)
 
     def write(self, data):
-        if self.channel is not None:
+        """Write data in asynchronous way."""
+        if self.channel is not None and not self.channel.closed:
             self.channel.write(data)
         else:
             self.stream.write(data)
 
-    def close(self):
-        if self.channel is not None and self.channel.active:
-            self.channel.stop()
-
-
-class Producers(dict):
-
-    def __getattr__(self, name):
-
-        def inner_function(*args, **kwargs):
-            results = []
-            for producer in self.values():
-                results.append(getattr(producer, name)(*args, **kwargs))
-            return results
-
-        inner_function.__name__ = name
-        return inner_function
-
 
 class ProcessManager(LogsMixin, LoopMixin):
     """Start and manage workers."""
@@ -68,33 +123,18 @@ class ProcessManager(LogsMixin, LoopMixin):
     gevent_monkey = 'from gevent.monkey import patch_all; patch_all();'
     script = 'from thriftpool.bin.thriftworker import main; main();'
 
-    def __init__(self, app, listeners, controller):
+    def __init__(self, app, listeners):
+        self.clients = Clients()
+        self.serializers = StreamSerializer()
         self.app = app
         self.listeners = listeners
-        self.producers = Producers()
-        self.controller = controller
-        self.bootstrapped = {}
+        self._bootstrapped = set()
         super(ProcessManager, self).__init__()
 
     @property
     def manager(self):
         return self.app.gaffer_manager
 
-    def _create_arguments(self):
-        worker_type = self.app.config.WORKER_TYPE
-        if worker_type == 'gevent':
-            startup_line = '{0} {1}'.format(self.gevent_monkey, self.script)
-        elif worker_type == 'sync':
-            startup_line = self.script
-        else:
-            raise NotImplementedError()
-        args = ['-c', '{0}'.format(startup_line)]
-        return dict(cmd=sys.executable, args=args,
-                    redirect_output=['out', 'err'],
-                    custom_streams=['handshake', 'incoming', 'outgoing'],
-                    custom_channels=self.listeners.channels,
-                    redirect_input=True)
-
     @cached_property
     def _is_started(self):
         return self.app.env.RealEvent()
@@ -111,48 +151,34 @@ class ProcessManager(LogsMixin, LoopMixin):
     def _stderr(self):
         return RedirectStream(self.loop, sys.stderr)
 
-    def _create_producer(self, process):
-        incoming = process.streams['incoming']
-        outgoing = process.streams['outgoing']
-        producer = self.producers[process.id] = \
-            Producer(self.loop, incoming, outgoing, process)
-        producer.start()
-        return producer
-
-    def _bootstrap_process(self, process):
-        # Now we can create producer.
-        producer = self._create_producer(process)
-
-        # Create name for process.
+    def _bootstrap_process(self, proxy, process):
+        # Change name of process.
         name = self.name_template.format(process.id, self.app.config)
+        proxy.change_title(name)
+
+        # Register acceptors in remote process.
+        proxy.register_acceptors({i: listener.name
+            for i, listener in iteritems(self.listeners.enumerated)})
 
         # Notify about process initialization.
-        def bootstrap_done(producer, reply):
-            self.bootstrapped[process.id] = producer
-            self._info('Worker %d initialized.', process.id)
-            state = self.manager.get_process(self.process_name)
-            if len(self.bootstrapped) >= state.numprocesses:
-                self._info('Workers initialization done.')
-                self._is_started.set()
-
-        # And bootstrap remote process.
-        producer.apply('change_title', args=[name])
-        descriptors = {i: listener.name
-                       for i, listener in iteritems(self.listeners.enumerated)}
-        producer.register_acceptors(descriptors).link(bootstrap_done)
+        self._bootstrapped.add(process.id)
+        self._info('Worker %d initialized.', process.id)
+        state = self.manager.get_process(self.process_name)
+        if len(self._bootstrapped) >= state.numprocesses:
+            self._info('Workers initialization done.')
+            self._is_started.set()
 
     def _do_handshake(self, process):
         # Pass application to created process.
         stream = process.streams['handshake']
-        message = pickle.dumps(self.app)
-        stream.write(struct.pack('I', len(message)) + message)
+        stream.write(self.serializers.encode_with_length(self.app))
 
         def handshake_done(evtype, info):
             stream.unsubscribe(handshake_done)
             # Process exited and we do the same.
             if not process.active:
                 return
-            self._bootstrap_process(process)
+            self.clients.register(process, self._bootstrap_process)
 
         # Wait for worker answer.
         stream.subscribe(handshake_done)
@@ -181,10 +207,6 @@ class ProcessManager(LogsMixin, LoopMixin):
             self._info('Worker %d spawned with pid %d.',
                        msg['pid'], msg['os_pid'])
 
-        if not self.controller.is_running:
-            # Controller not running, simply exit.
-            return
-
         if evtype == 'spawn':
             # New process spawned, handle event.
             process = self.manager.get_process(msg['pid'])
@@ -193,50 +215,72 @@ class ProcessManager(LogsMixin, LoopMixin):
 
         elif evtype == 'exit':
             # Process exited, handle event.
-            self.bootstrapped.pop(msg['pid'], None)
-            producer = self.producers.pop(msg['pid'], None)
-            if producer is not None:
-                producer.stop()
+            self._bootstrapped.remove(msg['pid'])
+            self.clients.unregister(msg['pid'])
+
+    def _create_proc_kwargs(self):
+        """Create arguments for worker."""
+        config = self.app.config
+        worker_type = config.WORKER_TYPE
+        if worker_type == 'gevent':
+            startup_line = '{0} {1}'.format(self.gevent_monkey, self.script)
+        elif worker_type == 'sync':
+            startup_line = self.script
+        else:
+            raise NotImplementedError()
+        return dict(cmd=sys.executable,
+                    args=['-c', '{0}'.format(startup_line)],
+                    redirect_output=['out', 'err'],
+                    custom_streams=['handshake', 'incoming', 'outgoing'],
+                    custom_channels=self.listeners.channels,
+                    env=dict(os.environ, IS_WORKER='1'),
+                    numprocesses=config.WORKERS,
+                    redirect_input=True)
+
+    def _create_apps(self):
+        """Create applications for gaffer."""
+        apps = []
+        options = dict(clients=self.clients)
+        handlers = [
+            (r'/timers', ClientsHandler, options),
+            (r'/timers/([0-9^/]+)', TimerHandler, options),
+            (r'/counters', ClientsHandler, options),
+            (r'/counters/([0-9^/]+)', CounterHandler, options),
+        ]
+        endpoints = self.app.config.TORNADO_ENDPOINTS
+        if endpoints:
+            apps.append(HttpHandler(handlers=handlers,
+                log_function=self.app.log.log_tornado_request,
+                endpoints=[HttpEndpoint(uri=uri) for uri in endpoints]))
+        return apps
 
     @in_loop
-    def _pre_start(self):
+    def _setup(self):
         manager = self.manager
         manager.subscribe('.', self._on_event)
-        manager.add_process(self.process_name,
-                            numprocesses=self.app.config.WORKERS,
-                            **self._create_arguments())
-        manager.start()
+        manager.add_process(self.process_name, **self._create_proc_kwargs())
+        manager.start(apps=self._create_apps())
 
     def start(self):
-        self._pre_start()
-        self._is_started.wait(30.0)
+        self._setup()
+        self._is_started.wait(START_TIMEOUT)
         if not self._is_started.is_set():
-            logger.error('Timeout when starting processes.')
+            self._error('Timeout when starting processes.')
+            self._teardown()
+            raise SystemTerminate()
 
     @in_loop
-    def _pre_stop(self):
-        for producer in self.producers.values():
-            producer.stop()
-        self.producers = {}
+    def _teardown(self):
+        self.clients.clear()
         stop_callback = lambda *args: self._is_stopped.set()
         self.manager.stop(stop_callback)
 
-    @in_loop
-    def _post_stop(self):
-        self._stderr.close()
-        self._stdout.close()
-
     def stop(self):
-        self._pre_stop()
-        self._is_stopped.wait(30.0)
+        self._teardown()
+        self._is_stopped.wait(STOP_TIMEOUT)
         if not self._is_stopped.is_set():
-            logger.error('Timeout when stopping processes.')
-        self._post_stop()
-
-    def apply(self, method_name, callback=None, args=None, kwargs=None):
-        """Run given method across all processes."""
-        for producer in self.producers.values():
-            producer.apply(method_name, callback, args, kwargs)
+            self._error('Timeout when terminating processes.')
+            raise SystemTerminate()
 
 
 class ProcessManagerComponent(StartStopComponent):
@@ -246,5 +290,5 @@ class ProcessManagerComponent(StartStopComponent):
 
     def create(self, parent):
         processes = parent.processes = \
-            ProcessManager(parent.app, parent.listeners, parent)
+            ProcessManager(parent.app, parent.listeners)
         return processes
